pub struct HandlerArgs { /* private fields */ }Expand description
Arguments passed to a handler.
Implementations
Sourceimpl HandlerArgs
impl HandlerArgs
Sourcepub fn new(ctx: HandlerCtx, req: Request) -> Self
pub fn new(ctx: HandlerCtx, req: Request) -> Self
Create new handler arguments.
Panics
If the ctx tracing span has not been initialized via
HandlerCtx::init_request_span.
Sourcepub fn into_parts(self) -> (HandlerCtx, Request)
pub fn into_parts(self) -> (HandlerCtx, Request)
Decompose the handler arguments into its parts.
Sourcepub const fn ctx(&self) -> &HandlerCtx
pub const fn ctx(&self) -> &HandlerCtx
Get a reference to the handler context.
Sourcepub fn span(&self) -> &Span
pub fn span(&self) -> &Span
Get a reference to the tracing span for this handler invocation.
Panics
If the span has not been initialized via
HandlerCtx::init_request_span.
Sourcepub fn otel_span_name(&self) -> String
pub fn otel_span_name(&self) -> String
Get the OpenTelemetry span name for this handler invocation.
Sourcepub const fn service_name(&self) -> &'static str
pub const fn service_name(&self) -> &'static str
Get the service name for this handler invocation.